
#ifndef NULL
  #define  NULL    ((void *)0)
#endif


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `BlockRangeRep': Block-distributed range.                     //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class BlockRangeRep : public RangeRep {
public :
  BlockRangeRep(int _extent, Dimension _dim, const int _blockSize) ;

  RangeRep* prm() ;

  int bas() ;
  int str() ;

  //int isScalar() ;

  Subscript* subscr(const int i) ;
  int subscr(const Subscript* s) ;

  RangeRep* subrng(const int extent, const int base, const int stride) ;

  StrideRep* vector(const int dataLen) ;

  virtual StrideRep* subrngVector(const int dataLen, const int stride) ;
      // Better ``protected''?

  void blockInit(Block& b) ;

  void locBlockInit(Block& b) ;

  void allBlocksEnum(Block& b) ;
  int  allBlocksTest(Block& b) ;
  void allBlocksNext(Block& b) ;

protected :
  int blockSize ;

private :
  void blockParams(Block& b, int i_lo, int i_hi) ;

MEMORY_POOL_DECS
} ;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `PackedBlockRangeRep': Block-distributed range yielding       //
//                              arrays with ``packed'' layout.         //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class PackedBlockRangeRep : public BlockRangeRep {
public :
  PackedBlockRangeRep(int _extent, Dimension _dim, const int _blockSize) ;

  StrideRep* subrngVector(const int dataLen, const int stride) ;

MEMORY_POOL_DECS
} ;

// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `ExtendedBlockRangeRep': Block-distributed range with         //
//                                yielding arrays with ghost edges.    //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class ExtendedBlockRangeRep : public BlockRangeRep {
public :
  ExtendedBlockRangeRep(int _extent, Dimension _dim, const int _blockSize,
                        const int _wlo, const int _whi) ;

  StrideRep* vector(const int dataLen) ;

private :
  int wlo, whi ;

MEMORY_POOL_DECS
} ;


// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //
//                                                                     //
// class `BlockSubRangeRep': range aligned to a                        //
//                           block-distributed range.                  //
//                                                                     //
// - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - //

class BlockSubRangeRep : public BlockRangeRep {
public :
  BlockSubRangeRep(const int _extent, const int _base, const int _stride,
                   BlockRangeRep* _parent,
                   Dimension _dim, const int _blockSize) ;
  ~BlockSubRangeRep() ;

  RangeRep* prm() ;
  int bas() ;
  int str() ;

  Subscript* subscr(const int i) ;
  int subscr(const Subscript* s) ;

  RangeRep* subrng(const int _extent, const int _base, const int _stride) ;

  StrideRep* vector(const int dataLen) ;

  void blockInit(Block& b) ;

  void locBlockInit(Block& b) ;

  void allBlocksEnum(Block& b) ;
  void allBlocksNext(Block& b) ;

protected :
  // Parent primitive range
  BlockRangeRep* parent ;

  // Alignment parameters
  int base ;
  int stride ;
private :
  void blockParams(Block& b, int i_lo, int i_hi) ;
  void blockRange(int& i_lo, int& i_hi, int _crd) ;

MEMORY_POOL_DECS
} ;


